#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>

typedef struct snd_ctl_card_info {
	int card;
	int pad;
	unsigned char id[16];
	unsigned char driver[16];
	unsigned char name[32];
	unsigned char longname[80];
	unsigned char reserved_[16];
	unsigned char mixername[80];
	unsigned char components[128];
} snd_ctl_card_info_t;

#define SNDRV_CTL_IOCTL_CARD_INFO _IOR('U', 0x01, snd_ctl_card_info_t)

int snd_determine_driver(int card, char **driver)
{
	char name[31];
    int fd = -1;
	snd_ctl_card_info_t info = {0};
	char *res = NULL;
	int err;

	assert(card <= SND_MAX_CARDS);
	snprintf(name, sizeof(name), "/dev/snd/controlC%u", card);
	name[sizeof(name)-1] = '\0';

	if (card >= SALSA_MAX_CARDS) {
		err = -EINVAL;
		goto __error;
    }

	fd = open(name, O_RDWR);
	if (fd < 0) {
        err = -errno;
		SNDERR("could not open control for card %u: %s", card, strerror(err));
		goto __error;
	}
	if (ioctl(fd, SNDRV_CTL_IOCTL_CARD_INFO, &info) < 0) {
		err = -errno;
		SNDERR("snd_ctl_card_info error: %s", strerror(err));
		goto __error;
	}
    res = malloc(33);
	if (res == NULL) {
		err = -ENOMEM;
	} else {
	    //res = _snd_strdup((const char*)info.driver);
        size_t len = strlen((const char*)info.driver);
        memcpy(res, (const char*)info.driver, len);
        res[len] = '\0';
		*driver = res;
		err = 0;
	}

__error:
	if (fd >= 0)
		close(fd);
	return err;
}

