/*
 *  SALSA-Lib - Input Macros
 *
 *  Copyright (c) 2007-2012 by Takashi Iwai <tiwai@suse.de>
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation; either version 2.1 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __ALSA_INPUT_H
#define __ALSA_INPUT_H

#include <stdio.h>
#include <errno.h>

typedef struct snd_input snd_input_t;

struct snd_input {
	int mode;
    char ungot[2];
	unsigned char *ptr;
	unsigned int pos;
	unsigned int size;
};

typedef enum _snd_input_type {
    SND_INPUT_BUFFER,
} snd_input_type_t;

__SALSA_EXPORT_FUNC
int snd_input_buffer_open(snd_input_t **inputp, const char *buffer, ssize_t size)
{
	snd_input_t *in = malloc(sizeof(*in));
	if (!in)
		return -ENOMEM;
	if (size < 0)
		size = strlen(buffer);
	in->mode = SND_INPUT_BUFFER;
	in->ptr = buffer;
	in->size = size;
    in->pos = 0;
    in->ungot[0] = in->ungot[1] = 0;
	*inputp = in;
	return 0;
}

__SALSA_EXPORT_FUNC
int snd_input_getc(snd_input_t *in)
{
    char c;

    if (in->ungot[1]) {
        c = in->ungot[0];
        in->ungot[1] = 0;
    } else {
        if (in->pos <= in->size) {
            c = in->ptr[in->pos++];
        } else {
            c = EOF;
        }
    }

    return c;
}

__SALSA_EXPORT_FUNC
int snd_input_ungetc(snd_input_t *in, int c)
{
    in->ungot[0] = c;
    in->ungot[1] = 1;
    return c;
}

__SALSA_EXPORT_FUNC
char *snd_input_buffer_gets(snd_input_t *in, char *str, size_t size)
{
	size_t bsize = in->size;
	while ((in->pos < in->size) && bsize > 0) {
		unsigned char c = in->ptr[in->pos++];
		bsize--;
		*str++ = c;
		if (c == '\n')
			break;
	}
	if (bsize == in->size)
		return NULL;
	*str = '\0';
	return str;
}
			
#define snd_input_close(input)      fclose(input)
#define snd_input_scanf         fscanf

__SALSA_EXPORT_FUNC __SALSA_NOT_IMPLEMENTED
int snd_input_stdio_open(snd_input_t **inputp, const char *file, const char *mode)
{
    return -ENXIO;
}

__SALSA_EXPORT_FUNC __SALSA_NOT_IMPLEMENTED
int snd_input_stdio_attach(snd_input_t **inputp, FILE *fp, int _close)
{
    return -ENXIO;
}
    
#endif /* __ALSA_INPUT_H */
