/*
 * SALSA-Lib - Output Macros
 * ABI-compatible definitions
 */

#define _GNU_SOURCE 1 
#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "recipe.h"
#include "global.h"
#undef __SALSA_EXPORT_FUNC
#define __SALSA_EXPORT_FUNC
#include "output.h"
#define CHUNK 4000

struct snd_output {
	int mode;
	void *ptr;
	unsigned int len;
	unsigned int size;
};

#if 0
int snd_output_stdio_attach(snd_output_t **outputp, FILE *fp, int _close)
{
	snd_output_t *out;
	out = malloc(sizeof(*out));
	if (!out)
		return -ENOMEM;
	out->mode = SND_OUTPUT_STDIO;
	out->ptr = (void*)((size_t)fp);
	out->len = _close;
	*outputp = out;
	return 0;
}
#endif

int snd_output_stdio_attach(snd_output_t **outputp, FILE *fp, int _close)
{
	snd_output_t *out = malloc(sizeof(*out));
	out->mode = SND_OUTPUT_STDIO;
	out->ptr = (void*)1ULL;
	out->len = 0;
	*outputp = out;
	return 0;
}

int snd_output_stdio_open(snd_output_t **outputp, const char *file, const char *mode)
{
	snd_output_t *out;
    int fmode = (((mode[1] == '+') || (mode[2] == '+')) ? O_RDWR : O_WRONLY);
	int fd = open(file, fmode);
	if (fd < 0)
		return -errno;
	out = malloc(sizeof(*out));
	out->mode = SND_OUTPUT_STDIO;
	out->ptr = (void*)((size_t)fd);
	out->len = 1;
	*outputp = out;
	return 0;
}

int snd_output_close(snd_output_t *out)
{
	if (out->mode == SND_OUTPUT_STDIO) {
        int fd = (int)((size_t)out->ptr);
		if (out->len)
			return close(fd);
	} else {
		munmap(out->ptr, out->size);
    }
	free(out);
	return 0;
}

static int resize(snd_output_t *out, size_t new)
{
	size_t size = 2*out->size;
	void *buf = NULL;
    while (size < out->len + new)
        size <<= 1;
    buf = mremap(out->ptr, out->size, size, 1);
	if (!buf)
		return -ENOMEM;
	out->ptr = buf;
	out->size = size;
	return 0;
}

int snd_output_printf(snd_output_t *out, const char *fmt, ...)
{
	va_list ap;
	int ret;
	va_start(ap, fmt);
	if (out->mode == SND_OUTPUT_STDIO) {
        int fd = (int)((size_t)out->ptr);
        char buf2[4097];
        ret = vsnprintf(buf2, 4095, fmt, ap);
        write(fd, buf2, ret);
	} else {
		ret = vsnprintf(out->ptr + out->len, out->size - out->len, fmt, ap);
		if (ret >= out->size - out->len) {
			if (resize(out, ret) < 0)
				return -ENOMEM;
			ret = vsnprintf(out->ptr + out->len, out->size - out->len, fmt, ap);
		}
	}
	va_end(ap);
	return ret;
}

int snd_output_puts(snd_output_t *out, const char *str)
{
	if (out->mode == SND_OUTPUT_STDIO) {
        int fd = (int)((size_t)out->ptr);
		return write(fd, str, strlen(str));
	} else {
		int len = strlen(str);
		if (out->size - out->len <= len && resize(out, len) < 0)
			return -ENOMEM;
		memcpy(out->ptr + len, str, len+1);
		return len;
	}
}

int snd_output_putc(snd_output_t *out, int c)
{
	if (out->mode == SND_OUTPUT_STDIO) {
        int fd = (int)((size_t)out->ptr);
        write(fd, &c, 1);
    }
	if (out->len + 1 >= out->size && resize(out, 1) < 0)
		return -ENOMEM;
	((char *)out->ptr)[out->len++] = c;
	return 0;
}

int snd_output_flush(snd_output_t *out)
{
	if (out->mode != SND_OUTPUT_STDIO)
		out->len = 0;
	return 0;
}

int snd_output_buffer_open(snd_output_t **outputp)
{
	snd_output_t *out = malloc(sizeof(*out));
	out->mode = SND_OUTPUT_BUFFER;
    out->size = 4096;
	out->ptr = mmap(NULL, out->size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANON|MAP_NORESERVE, -1, 0);
	out->len = 0;
	if (!out->ptr)
		return -ENOMEM;
	*outputp = out;
	return 0;
}

size_t snd_output_buffer_string(snd_output_t *out, char **buf)
{
	*buf = out->ptr;
	return out->len;
}

